"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DBManager = void 0;
const application_1 = require("../tools/application");
const noomierror_1 = require("../tools/noomierror");
const instancefactory_1 = require("../main/instancefactory");
const transactionmanager_1 = require("./transactionmanager");
const threadlocal_1 = require("../tools/threadlocal");
/**
 * 数据库管理器
 * @remarks
 * 用于管理数据库相关配置
 */
class DBManager {
    /**
     * 初始化
     * @param cfg -   配置项,参考数据库配置
     */
    static init(cfg) {
        if (!cfg.package) {
            throw new noomierror_1.NoomiError("4001");
        }
        //从配置的noomi数据库包获取transaction和connection manager类
        const { Transaction, ConnectionManager } = require(cfg.package);
        if (!Transaction || !ConnectionManager) {
            throw new noomierror_1.NoomiError("4002", cfg.package);
        }
        //记录connection 和 transaction class
        this.connectionManagerClazz = ConnectionManager;
        this.transactionClazz = Transaction;
        //添加connection manager到InstanceFactory
        instancefactory_1.InstanceFactory.addInstance(this.connectionManagerClazz, {
            singleton: true,
            params: [cfg.options]
        });
        //添加transaction class 到InstanceFactory
        instancefactory_1.InstanceFactory.addInstance(this.transactionClazz, false);
        //事务类初始化
        transactionmanager_1.TransactionManager.initAdvice();
    }
    /**
     * 解析文件
     * @param path -  文件路径
     */
    static parseFile(path) {
        try {
            const jsonStr = application_1.App.fs.readFileSync(path, 'utf-8');
            const json = application_1.App.JSON.parse(jsonStr);
            this.init(json);
        }
        catch (e) {
            throw new noomierror_1.NoomiError("2800") + '\n' + e;
        }
    }
    /**
     * 获取连接
     * @returns     connection对象(具体connection对象由package确定)
     */
    static async getConnection() {
        const id = this.getAsyncId();
        const obj = this.connectionMap.get(id);
        if (obj && obj.conn) {
            return obj.conn;
        }
        //从package对应包中获取connection
        const conn = await this.connectionManagerClazz.getConnection();
        //存储conn
        this.connectionMap.get(id).conn = conn;
        return conn;
    }
    /**
     * 释放连接
     */
    static async closeConnection() {
        const id = this.getAsyncId();
        const obj = this.connectionMap.get(id);
        if (!obj || !obj.conn) {
            return;
        }
        if (obj.tx) {
            obj.tx.commit();
        }
        //调用实际conn关闭方法
        await instancefactory_1.InstanceFactory.getInstance(this.connectionManagerClazz).closeConnection(obj.conn);
        //从map移除
        this.connectionMap.delete(id);
    }
    /**
     * 获取事务，类型由具体package确定
     * @returns     事务对象
     */
    static async getTransaction() {
        const id = this.getAsyncId();
        let tx = this.connectionMap.get(id).tx;
        if (!tx) {
            tx = instancefactory_1.InstanceFactory.getInstance(this.transactionClazz);
            this.connectionMap.get(id).tx = tx;
        }
        return tx;
    }
    /**
     * 移除事务
     */
    static async removeTransaction() {
        const id = threadlocal_1.NoomiThreadLocal.getThreadId();
        if (id && this.connectionMap.has(id)) {
            delete this.connectionMap.get(id).tx;
        }
    }
    /**
     * 获取异步id
     * @returns     异步id
     */
    static getAsyncId() {
        let id = threadlocal_1.NoomiThreadLocal.getThreadId();
        if (!id) {
            id = threadlocal_1.NoomiThreadLocal.newThreadId();
        }
        if (!this.connectionMap.has(id)) {
            this.connectionMap.set(id, {});
        }
        return id;
    }
}
exports.DBManager = DBManager;
/**
 * connection map
 * @remarks
 * 存放connection，键为threadid，值为object，格式为
 * ```json
 * {
 *      conn:连接对象
 *      tr:事务对象
 *      manager: entitymanager
 * }
 * ```
 */
DBManager.connectionMap = new Map();
//# sourceMappingURL=dbmanager.js.map